"""Delineate drainage basin using NLDI and user-defined point."""
import geojson

from .utils import check_coords
from .utils import geom_to_geojson
from .utils import get_coordsys
from .utils import get_local_catchment
from .utils import get_local_flowline
from .utils import get_on_flowline
from .utils import get_total_basin
from .utils import get_upstream_basin
from .utils import JsonFeatureCollectionType
from .utils import merge_geometry
from .utils import project_point
from .utils import split_catchment


class SplitCatchment:
    """Define inputs and outputs for the main SplitCatchment class."""

    def __init__(self: "SplitCatchment", x: float, y: float, upstream: bool) -> None:
        """Initialize Splitcatchment class."""
        self.x = x
        self.y = y
        self.catchmentIdentifier: str
        self.flowline: JsonFeatureCollectionType
        self.flw = None
        self.flwdir_transform: object
        self.projected_xy = None
        self.onFlowline: bool
        self.upstream = upstream

        # geoms
        self.catchmentGeom = None
        self.splitCatchmentGeom = None
        self.totalBasinGeom = None
        self.upstreamBasinGeom = None
        self.mergedCatchmentGeom = None
        self.nhdFlowlineGeom = None

        # outputs
        self.catchment: JsonFeatureCollectionType
        self.splitCatchment: JsonFeatureCollectionType
        self.upstreamBasin: JsonFeatureCollectionType
        self.mergedCatchment: JsonFeatureCollectionType

        # create transform
        self.transformToRaster: object
        self.transformToWGS84: object

        # kick off
        self.run()

    def serialize(self) -> geojson.feature.FeatureCollection:
        """Convert returns to GeoJSON to be exported."""
        # If upstream == False, only return the local catchment
        # and the splitcatchment geometries
        if self.upstream is False:
            feature1 = geojson.Feature(
                geometry=self.catchment,
                id="catchment",
                properties={"catchmentID": self.catchmentIdentifier},
            )
            feature2 = geojson.Feature(
                geometry=self.splitCatchment, id="splitCatchment"
            )
            featurecollection = geojson.FeatureCollection([feature1, feature2])

        # If upstream == True and the clickpoint is on a NHD FLowline,
        # return the local catchment and the merged catchment
        # (splitcatchment merged with all upstream basins)
        elif self.upstream is True and self.onFlowline is True:
            feature1 = geojson.Feature(
                geometry=self.catchment,
                id="catchment",
                properties={"catchmentID": self.catchmentIdentifier},
            )
            feature2 = geojson.Feature(
                geometry=self.mergedCatchment, id="mergedCatchment"
            )
            featurecollection = geojson.FeatureCollection([feature1, feature2])

        # If upstream == True and the clickpoint is NOT on a NHD FLowline,
        # return the local catchment and splitcatchment
        elif self.upstream is True and self.onFlowline is False:
            feature1 = geojson.Feature(
                geometry=self.catchment,
                id="catchment",
                properties={"catchmentID": self.catchmentIdentifier},
            )
            feature2 = geojson.Feature(
                geometry=self.splitCatchment, id="splitCatchment"
            )
            # feature3 = geojson.Feature(geometry=self.upstreamBasin, id='upstreamBasin')
            featurecollection = geojson.FeatureCollection([feature1, feature2])

        return featurecollection

    # main functions
    def run(self) -> None:
        """Run splitcatchment module functions."""
        # Order of these functions is important!
        check_coords(self.x, self.y)
        self.catchmentIdentifier, self.catchmentGeom = get_local_catchment(
            self.x, self.y
        )
        self.flowline, self.nhdFlowlineGeom = get_local_flowline(
            self.catchmentIdentifier
        )
        self.transformToRaster, self.transformToWGS84 = get_coordsys()
        self.projected_xy = project_point(self.x, self.y, self.transformToRaster)
        self.splitCatchmentGeom = split_catchment(
            self.catchmentGeom,
            self.projected_xy,
            self.transformToRaster,
            self.transformToWGS84,
        )
        self.onFlowline = get_on_flowline(
            self.projected_xy, self.flowline, self.transformToRaster
        )
        self.catchment = geom_to_geojson(self.catchmentGeom)

        # outputs
        if self.upstream is False:
            self.splitCatchment = geom_to_geojson(self.splitCatchmentGeom)

        if self.upstream is True and self.onFlowline is True:
            self.totalBasinGeom = get_total_basin(self.catchmentIdentifier)
            self.mergedCatchmentGeom = merge_geometry(
                self.catchmentGeom, self.splitCatchmentGeom, self.totalBasinGeom
            )
            self.mergedCatchment = geom_to_geojson(self.mergedCatchmentGeom)

        if self.upstream is True and self.onFlowline is False:
            self.splitCatchment = geom_to_geojson(self.splitCatchmentGeom)
            self.totalBasinGeom = get_total_basin(self.catchmentIdentifier)
            self.upstreamBasinGeom = get_upstream_basin(
                self.catchmentGeom, self.totalBasinGeom
            )
            self.upstreamBasin = geom_to_geojson(self.upstreamBasinGeom)



"""Pygeoapi instance of NLDI get splitcatchment at point."""
import logging
import time
from distutils.util import strtobool
from typing import Any

from pygeoapi.process.base import BaseProcessor

from nldi_flowtools.splitcatchment import SplitCatchment

LOGGER = logging.getLogger(__name__)

PROCESS_METADATA = {
    "version": "0.1.0",
    "id": "nldi-splitcatchment",
    "title": "NLDI Split Catchment process",
    "description": "NLDI Split Catchment process",
    "keywords": ["NLDI Split Catchment"],
    "links": [
        {
            "type": "text/html",
            "rel": "canonical",
            "title": "information",
            "href": "https://example.org/process",
            "hreflang": "en-US",
        }
    ],
    "inputs": [
        {
            "id": "lat",
            "title": "lat",
            "abstract": "The latitude coordinate of the pour point in decimal degrees",
            "input": {
                "literalDataDomain": {
                    "dataType": "float",
                    "valueDefinition": {"anyValue": True},
                }
            },
            "minOccurs": 1,
            "maxOccurs": 1,
        },
        {
            "id": "lon",
            "title": "lon",
            "abstract": "The longitude coordinate of the pour point in decimal degrees",
            "input": {
                "literalDataDomain": {
                    "dataType": "float",
                    "valueDefinition": {"anyValue": True},
                }
            },
            "minOccurs": 1,
            "maxOccurs": 1,
        },
        {
            "id": "upstream",
            "title": "upstream",
            "abstract": "Determines whether to return the portion of the drainage basin "
            "that falls outside of the local catchment. If True, then the entire "
            "drainage basin is returned. If False, then only the portion within the local "
            "catchment is returned.",
            "input": {
                "literalDataDomain": {
                    "dataType": "boolean",
                    "valueDefinition": {"anyValue": True},
                }
            },
            "minOccurs": 1,
            "maxOccurs": 1,
        },
    ],
    "outputs": [
        {
            "id": "catchment",
            "title": "catchment",
            "description": "The local NHD catchment that the pour point falls within."
            " This is also the catchment that gets split.",
            "output": {"formats": [{"mimeType": "application/geo+json"}]},
        },
        {
            "id": "splitCatchment",
            "title": "splitCatchment",
            "description": "Either a portion or the entire drainage basin for the pour "
            "point, depending if the pour point falls on an NHD flowline or not. It "
            "gets returned if the drainage basin fits only within the local catchment, "
            "or if the upstream variable is set to False.",
            "output": {"formats": [{"mimeType": "application/geo+json"}]},
        },
        {
            "id": "mergedCatchment",
            "title": "mergedCatchment",
            "description": "The entire drainage basin which flows to the pour point. "
            "It will include area outside of the local catchment, and it will only be "
            "returned if the upstream variable is set to True.",
            "output": {"formats": [{"mimeType": "application/geo+json"}]},
        },
    ],
    "example": {
        "inputs": [
            {"id": "lat", "value": "43.29139", "type": "text/plain"},
            {"id": "lon", "value": "-73.82705", "type": "text/plain"},
            {"id": "upstream", "value": "False", "type": "text/plain"},
        ]
    },
}


class NLDISplitCatchmentProcessor(BaseProcessor):  # type: ignore
    """NLDI Split Catchment Processor."""

    def __init__(self, provider_def: Any):
        """Initialize object.

        :param provider_def: provider definition
        :returns: pygeoapi.process.nldi_delineate.NLDIDelineateProcessor
        """
        BaseProcessor.__init__(self, provider_def, PROCESS_METADATA)

    def execute(self, data: Any) -> tuple[str, dict[Any, Any]]:
        """Execute Split Catchment Processor."""
        mimetype = "application/json"
        newdata: dict[str, str] = {d["id"]: d["value"] for d in data}
        print(newdata)
        lat = float(newdata.get("lat"))
        lon = float(newdata.get("lon"))
        upstream = bool(strtobool(newdata.get("upstream")))

        print(lat, lon, upstream)

        time_before = time.perf_counter()

        results = SplitCatchment(lon, lat, upstream)

        time_after = time.perf_counter()
        total_time = time_after - time_before
        print("Total Time:", total_time)

        # outputs = [{
        #     'results': results.serialize()
        # }]

        return mimetype, results.serialize()

    def __repr__(self) -> str:  # noqa D105
        return "<NLDISplitCatchmentProcessor> {}".format(
            self.nldi - splitcatchment - response  # type: ignore  # noqa F821
        )
